/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://java.net/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.observation.simple.rdf;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import it.tidalwave.util.As;
import it.tidalwave.role.spi.DefaultDisplayable;
import it.tidalwave.role.spi.DefaultIdentifiable;
import it.tidalwave.semantic.io.spi.StatementUnmarshallerSupport;
import it.tidalwave.observation.Location;
import it.tidalwave.observation.ObservationItem;
import it.tidalwave.observation.Observer;
import it.tidalwave.observation.Source;
import it.tidalwave.observation.simple.SimpleObservation;
import it.tidalwave.observation.simple.SimpleObservationItem;
import it.tidalwave.observation.simple.SimpleObservationSet;
import it.tidalwave.observation.simple.SimpleObserver;
import it.tidalwave.semantic.io.impl.Converter;
import org.openrdf.model.Statement;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class SimpleObservationUnmarshaller extends StatementUnmarshallerSupport
  {
    @Override @Nonnull
    public As unmarshal (final @Nonnull List<Statement> statements, final @Nonnull Context context)
      {
        final SimpleObservationSet observationSet = (SimpleObservationSet)context.peek();
        final Date date = (Date)getLiteral(statements, Vocabulary.DC_DATE);
        final Location location = (Location)findEntity(context, statements, Vocabulary.MADE_AT);
        final Source source = (Source)findEntity(context, statements, Vocabulary.DC_PUBLISHER);
        final Set<Observer> observers = new HashSet<Observer>();
        final Set<ObservationItem> observationItems = new HashSet<ObservationItem>();
        final List<Object> extras = new ArrayList<Object>();
        extras.add(new DefaultIdentifiable(Converter.valueToId(statements.get(0).getSubject())));

        final String displayName = (String)getLiteral(statements, Vocabulary.RDFS_LABEL);

        if (displayName != null)
          {
            extras.add(new DefaultDisplayable(displayName, "SimpleObservation"));
          }

        for (final Statement statement : context.findStatements(Vocabulary.FOAF_TOPIC, Converter.valueToId(statements.get(0).getSubject())))
          {
            extras.add(context.find(statement.getSubject()));
          }

        final SimpleObservation observation = new SimpleObservation(observationSet, date, location, observers, observationItems, source, extras);
        context.push(observation);

        for (final Statement statement : findStatementsWithPredicate(statements, Vocabulary.SKOS_NARROWER))
          {
            observationItems.add((SimpleObservationItem)context.find(statement.getObject()));
          }

        for (final Statement statement : findStatementsWithPredicate(statements, Vocabulary.DC_CREATOR))
          {
            observers.add((SimpleObserver)context.find(statement.getObject()));
          }

        context.pop();

        return observation;
      }
  }
